/**
 * workflowservice.js
 * Workflow Service
 */
'use strict';
const edge = require('edge');

/**
 *  Represents a Workflow Service
 *  @class WorkflowService
 */
class WorkflowService {

  /**
   * Creates a new instance of WorkflowService.
   * @constructor
   * @param {Object} config A service configuration.
   */
  constructor (config) {
    this._config = config;

    // Load the workflow service assembly 
    this._createWorkflowServiceFacade = edge.func({
      assemblyFile: this._config.serviceAssemblyFilePath,
      typeName: 'Advantys.Workflow.ServiceFacade',
      methodName: 'CreateProxyAsync'
    });

  }

  /**
   * Starts the workflow service.
   * @return {Promise<Object>} A promise.
   */
  start () {
    const self = this;

    return new Promise(function (resolve, reject) {
      self._createWorkflowServiceFacade(self._config, function (error, result) {

        // HACK: Without it: side effects on Promises. 
        // https://github.com/tjanczuk/edge/issues/485
        setImmediate(() => {
        });

        if (error) {
          if (self._config.debugEnabled) {
            self._config.logHandler("Workflow Service: start: error: " + error);
          }
          reject(error);
        } else {
          self._workflowFacade = result;
          resolve();
        }
      });
    });
  }

  /**
   * Invokes a workfow operation asynchronously.
   * @param {String} operation An operation to execute.
   * @param {Object} context A workflow context.
   * @param {Object} args Arguments.
   * @return {Promise<Object>} A promise.
   */
  invoke (operation, context, args) {
    const self = this;

    if (!self._workflowFacade) {
      throw ('Workflow Service is not started');
    }

    return new Promise(function (resolve, reject) {
      self._workflowFacade.invoke(
        {
          operation: operation,
          context: context,
          args: args
        },
        function (error, result) {
        
          // HACK: Without it: It hangs. 
          // But some side effects on Promises and Data Loaders. 
          // https://github.com/tjanczuk/edge/issues/485
          setImmediate(() => {
          });

          if (error || result.error) {
            if (self._config.debugEnabled) {
              self._config.logHandler("Workflow Service: invoke: error: " + (error || result.error));
            }
            reject(error || result.error);
          } else {
            resolve(result.data);
          }
        });
    });
  }

}

module.exports = WorkflowService;
